/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Copyright (C) 2018 MediaTek Inc.
 */

#ifndef __MEDIATEK_INIT_H_
#define __MEDIATEK_INIT_H_

extern int mtk_soc_early_init(void);

#endif /* __MEDIATEK_INIT_H_ */
