/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Copyright (C) 2015 Renesas Electronics Europe GmbH
 * Copyright (C) 2015 Renesas Electronics Corporation
 * Copyright (C) 2015 Cogent Embedded, Inc.
 */

#ifndef __QOS_H__
#define __QOS_H__

void qos_init(void);

#endif
