/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * (C) Copyright 2016 Rockchip Electronics Co., Ltd
 */

#ifndef __EVB_RK3328_H
#define __EVB_RK3328_H

#include <configs/rk3328_common.h>

#define SDRAM_BANK_SIZE			(2UL << 30)

#endif
