/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * (C) Copyright 2019 Luca Ceresoli <luca@lucaceresoli.net>
 *
 * Declaration of PMU config object binary blob linked in at build time.
 */

extern const u32 zynqmp_pm_cfg_obj[];
extern const int zynqmp_pm_cfg_obj_size;
